@@ -1,3 +1,4 @@
+use crate::ast::Identifier;
 use crate::kalk_num::KalkNum;
 use crate::{
     ast::{Expr, Stmt},
@@ -29,6 +30,7 @@ pub struct Context {
     parsing_identifier_stmt: bool,
     equation_variable: Option<String>,
     contains_equal_sign: bool,
+    is_in_integral: bool,
 }
 
 #[wasm_bindgen]
@@ -46,6 +48,7 @@ impl Context {
             parsing_identifier_stmt: false,
             equation_variable: None,
             contains_equal_sign: false,
+            is_in_integral: false,
         };
 
         parse(&mut context, crate::prelude::INIT).unwrap();
@@ -90,6 +93,7 @@ impl Default for Context {
 /// Error that occured during parsing or evaluation.
 #[derive(Debug, Clone, PartialEq)]
 pub enum CalcError {
+    ExpectedDx,
     IncorrectAmountOfArguments(usize, String, usize),
     InvalidNumberLiteral(String),
     InvalidOperator,
@@ -108,6 +112,7 @@ pub enum CalcError {
 impl ToString for CalcError {
     fn to_string(&self) -> String {
         match self {
+            CalcError::ExpectedDx => format!("Expected eg. dx, to specify for which variable the operation is being done to. Example with integration: ∫(0, 1, x dx)."),
             CalcError::IncorrectAmountOfArguments(expected, func, got) => format!(
                 "Expected {} arguments for function {}, but got {}.",
                 expected, func, got
@@ -201,7 +206,7 @@ fn parse_identifier_stmt(context: &mut Context) -> Result<Stmt, CalcError> {
     if let TokenKind::Equals = peek(context).kind {
         // Use the "function call" expression that was parsed, and put its values into a function declaration statement instead.
         if let Expr::FnCall(identifier, parameters) = primary {
-            if !prelude::is_prelude_func(&identifier) {
+            if !prelude::is_prelude_func(&identifier.full_name) {
                 advance(context);
                 let expr = parse_expr(context)?;
                 let mut parameter_identifiers = Vec::new();
@@ -211,7 +216,7 @@ fn parse_identifier_stmt(context: &mut Context) -> Result<Stmt, CalcError> {
                 // Extract these.
                 for parameter in parameters {
                     if let Expr::Var(parameter_identifier) = parameter {
-                        parameter_identifiers.push(parameter_identifier);
+                        parameter_identifiers.push(parameter_identifier.full_name);
                     }
                 }
 
@@ -241,7 +246,10 @@ fn parse_var_decl_stmt(context: &mut Context) -> Result<Stmt, CalcError> {
         return Err(CalcError::VariableReferencesItself);
     }
 
-    Ok(Stmt::VarDecl(identifier.value, Box::new(expr)))
+    Ok(Stmt::VarDecl(
+        Identifier::from_full_name(&identifier.value),
+        Box::new(expr),
+    ))
 }
 
 fn parse_unit_decl_stmt(context: &mut Context) -> Result<Stmt, CalcError> {
@@ -305,9 +313,10 @@ fn parse_equation(context: &mut Context) -> Result<Expr, CalcError> {
             return Err(CalcError::UnableToSolveEquation);
         }
 
-        context
-            .symbol_table
-            .insert(Stmt::VarDecl(var_name.into(), Box::new(inverted.clone())));
+        context.symbol_table.insert(Stmt::VarDecl(
+            Identifier::from_full_name(var_name),
+            Box::new(inverted.clone()),
+        ));
         return Ok(inverted);
     }
 
@@ -319,7 +328,7 @@ fn parse_to(context: &mut Context) -> Result<Expr, CalcError> {
 
     if match_token(context, TokenKind::ToKeyword) {
         advance(context);
-        let right = Expr::Var(advance(context).value.clone()); // Parse this as a variable for now.
+        let right = Expr::Var(Identifier::from_full_name(&advance(context).value)); // Parse this as a variable for now.
 
         return Ok(Expr::Binary(
             Box::new(left),
@@ -465,24 +474,28 @@ fn parse_group_fn(context: &mut Context) -> Result<Expr, CalcError> {
     let expr = parse_expr(context)?;
     advance(context);
 
-    Ok(Expr::FnCall(name.to_string(), vec![expr]))
+    Ok(Expr::FnCall(Identifier::from_full_name(name), vec![expr]))
 }
 
 fn parse_identifier(context: &mut Context) -> Result<Expr, CalcError> {
-    let identifier = advance(context).clone();
+    let identifier = Identifier::from_full_name(&advance(context).value);
 
     // Eg. sqrt64
-    if match_token(context, TokenKind::Literal) {
+    if match_token(context, TokenKind::Literal)
+        || peek(context).value == "π"
+        || peek(context).value == "τ"
+        || peek(context).value == "ϕ"
+    {
         // If there is a function with this name, parse it as a function, with the next token as the argument.
-        if context.symbol_table.contains_fn(&identifier.value) {
-            let parameter = Expr::Literal(string_to_num(&advance(context).value));
-            return Ok(Expr::FnCall(identifier.value, vec![parameter]));
+        if context.symbol_table.contains_fn(&identifier.pure_name) {
+            let parameter = parse_primary(context)?;
+            return Ok(Expr::FnCall(identifier, vec![parameter]));
         }
     }
 
     let parse_as_var_instead = match_token(context, TokenKind::OpenParenthesis)
         && !context.parsing_identifier_stmt
-        && !context.symbol_table.contains_fn(&identifier.value);
+        && !context.symbol_table.contains_fn(&identifier.pure_name);
 
     // Eg. sqrt(64)
     // If the function doesn't exist, parse it as a variable and multiplication instead.
@@ -491,6 +504,11 @@ fn parse_identifier(context: &mut Context) -> Result<Expr, CalcError> {
     if !parse_as_var_instead && match_token(context, TokenKind::OpenParenthesis) {
         advance(context);
 
+        let is_integral = identifier.full_name == "integrate" || identifier.full_name == "∫";
+        if is_integral {
+            context.is_in_integral = true;
+        }
+
         let mut parameters = Vec::new();
         parameters.push(parse_expr(context)?);
 
@@ -501,27 +519,38 @@ fn parse_identifier(context: &mut Context) -> Result<Expr, CalcError> {
 
         consume(context, TokenKind::ClosedParenthesis)?;
 
-        return Ok(Expr::FnCall(identifier.value, parameters));
+        if is_integral {
+            context.is_in_integral = false;
+        }
+
+        return Ok(Expr::FnCall(identifier, parameters));
+    }
+
+    // Eg. dx inside an integral, should be parsed as *one* identifier
+    if context.is_in_integral && identifier.full_name.starts_with("d") {
+        return Ok(Expr::Var(identifier));
     }
 
     // Eg. x
-    if parse_as_var_instead || context.symbol_table.contains_var(&identifier.value) {
-        Ok(Expr::Var(identifier.value))
+    if parse_as_var_instead || context.symbol_table.contains_var(&identifier.pure_name) {
+        Ok(Expr::Var(identifier))
     } else if context.parsing_unit_decl {
-        context.unit_decl_base_unit = Some(identifier.value);
-        Ok(Expr::Var(DECL_UNIT.into()))
+        context.unit_decl_base_unit = Some(identifier.full_name);
+        Ok(Expr::Var(Identifier::from_full_name(DECL_UNIT)))
     } else {
         if let Some(equation_var) = &context.equation_variable {
-            if &identifier.value == equation_var {
-                return Ok(Expr::Var(identifier.value));
+            if &identifier.full_name == equation_var {
+                return Ok(Expr::Var(identifier));
             }
         } else if context.contains_equal_sign {
-            context.equation_variable = Some(identifier.value.clone());
-            return Ok(Expr::Var(identifier.value));
+            context.equation_variable = Some(identifier.full_name.clone());
+            return Ok(Expr::Var(identifier));
         }
 
-        let mut chars = identifier.value.chars();
-        let mut left = Expr::Var(chars.next().unwrap().to_string());
+        let mut chars = identifier.pure_name.chars();
+        let mut left = Expr::Var(Identifier::from_full_name(
+            &chars.next().unwrap().to_string(),
+        ));
 
         // Turn each individual character into its own variable reference.
         // This parses eg `xy` as `x*y` instead of *one* variable.
@@ -538,12 +567,13 @@ fn parse_identifier(context: &mut Context) -> Result<Expr, CalcError> {
 
                 parse_exponent(context)?
             } else {
-                Expr::Var(c.to_string())
+                Expr::Var(Identifier::from_full_name(&c.to_string()))
             };
 
             left = Expr::Binary(Box::new(left), TokenKind::Star, Box::new(right));
         }
 
+        // TODO: When implementing derivation for variables, make sure to add the derivation here.
         Ok(left)
     }
 }
@@ -592,6 +622,7 @@ fn string_to_num(value: &str) -> f64 {
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::ast::Identifier;
     use crate::lexer::{Token, TokenKind::*};
     use crate::test_helpers::*;
     use wasm_bindgen_test::*;
@@ -745,7 +776,7 @@ mod tests {
         assert_eq!(
             parse(tokens).unwrap(),
             Stmt::VarDecl(
-                String::from("x"),
+                Identifier::from_full_name("x"),
                 binary(literal(1f64), Plus, literal(2f64))
             )
         );
@@ -769,7 +800,7 @@ mod tests {
         assert_eq!(
             parse(tokens).unwrap(),
             Stmt::FnDecl(
-                String::from("f"),
+                Identifier::from_full_name("f"),
                 vec![String::from("x")],
                 binary(literal(1f64), Plus, literal(2f64))
             )
@@ -795,7 +826,7 @@ mod tests {
 
         // Add the function to the symbol table first, in order to prevent errors.
         context.symbol_table.set(Stmt::FnDecl(
-            String::from("f"),
+            Identifier::from_full_name("f"),
             vec![String::from("x")],
             literal(1f64),
         ));
@@ -804,7 +835,7 @@ mod tests {
             parse_with_context(&mut context, tokens).unwrap(),
             Stmt::Expr(binary(
                 Box::new(Expr::FnCall(
-                    String::from("f"),
+                    Identifier::from_full_name("f"),
                     vec![*binary(literal(1f64), Plus, literal(2f64))]
                 )),
                 Plus,
